@@ -8,6 +8,7 @@
 import copy
 import math
 import os
+import pathlib
 import pickle
 import sys
 
@@ -19,7 +20,7 @@ from blosc2 import blosc2_ext
 
 def _check_typesize(typesize):
     if not 1 <= typesize <= blosc2_ext.MAX_TYPESIZE:
-        raise ValueError("typesize can only be in the 1-%d range." % blosc2_ext.MAX_TYPESIZE)
+        raise ValueError(f"typesize can only be in the 1-{blosc2_ext.MAX_TYPESIZE} range.")
 
 
 def _check_clevel(clevel):
@@ -29,19 +30,19 @@ def _check_clevel(clevel):
 
 def _check_input_length(input_name, input_len, typesize, _ignore_multiple_size=False):
     if input_len > blosc2_ext.MAX_BUFFERSIZE:
-        raise ValueError("%s cannot be larger than %d bytes" % (input_name, blosc2_ext.MAX_BUFFERSIZE))
+        raise ValueError(f"{input_name} cannot be larger than {blosc2_ext.MAX_BUFFERSIZE} bytes")
     if not _ignore_multiple_size and input_len % typesize != 0:
-        raise ValueError("len(%s) can only be a multiple of typesize (%d)." % (input_name, typesize))
+        raise ValueError(f"len({input_name}) can only be a multiple of typesize ({typesize}).")
 
 
 def _check_filter(filter):
     if filter not in blosc2.Filter:
-        raise ValueError("filter can only be one of ", blosc2.Filter.keys())
+        raise ValueError(f"filter can only be one of: {blosc2.Filter.keys()}")
 
 
 def _check_codec(codec):
     if codec not in blosc2.Codec:
-        raise ValueError("codec can only be one of: %s, not '%s'" % (codecs, codec))
+        raise ValueError(f"codec can only be one of: {codecs}, not '{codec}'")
 
 
 def compress(
@@ -88,7 +89,7 @@ def compress(
     -----
     The `cname` and `shuffle` parameters in python-blosc API have been replaced by :paramref:`codec` and
     :paramref:`filter` respectively.
-    To set :paramref:`codec` and :paramref:`fitler`, the enumerates :class:`Codec` and :class:`Filter`
+    To set :paramref:`codec` and :paramref:`filter`, the enumerates :class:`Codec` and :class:`Filter`
     have to be used instead of the python-blosc API variables such as `blosc.SHUFFLE` for :paramref:`filter`
     or strings like "blosclz" for :paramref:`codec`.
 
@@ -222,7 +223,7 @@ def pack(obj, clevel=9, filter=blosc2.Filter.SHUFFLE, codec=blosc2.Codec.BLOSCLZ
     -----
     The `cname` and `shuffle` parameters in python-blosc API have been replaced by :paramref:`codec` and
     :paramref:`filter` respectively.
-    To set :paramref:`codec` and :paramref:`fitler`, the enumerates :class:`Codec` and :class:`Filter`
+    To set :paramref:`codec` and :paramref:`filter`, the enumerates :class:`Codec` and :class:`Filter`
     have to be used instead of the python-blosc API variables such as `blosc.SHUFFLE` for :paramref:`filter`
     or strings like "blosclz" for :paramref:`codec`.
 
@@ -600,8 +601,9 @@ def pack_tensor(tensor, chunksize=None, **kwargs):
     >>> import numpy as np
     >>> th = np.arange(1e6, dtype=np.float32)
     >>> cframe = blosc2.pack_tensor(th)
-    >>> len(cframe) < th.size * th.itemsize
-    True
+    >>> if not os.getenv("BTUNE_TRADEOFF"):
+    ...     assert len(cframe) < th.size * th.itemsize
+    ...
 
     See also
     --------
@@ -732,8 +734,9 @@ def save_tensor(tensor, urlpath, chunksize=None, **kwargs):
     >>> import numpy as np
     >>> th = np.arange(1e6, dtype=np.float32)
     >>> serial_size = blosc2.save_tensor(th, "test.bl2", mode="w")
-    >>> serial_size < th.size * th.itemsize
-    True
+    >>> if not os.getenv("BTUNE_TRADEOFF"):
+    ...     assert serial_size < th.size * th.itemsize
+    ...
 
     See also
     --------
@@ -769,8 +772,9 @@ def load_tensor(urlpath):
     >>> import numpy as np
     >>> th = np.arange(1e6, dtype=np.float32)
     >>> size = blosc2.save_tensor(th, "test.bl2", mode="w")
-    >>> size < th.size * th.itemsize
-    True
+    >>> if not os.getenv("BTUNE_TRADEOFF"):
+    ...     assert size < th.size * th.itemsize
+    ...
     >>> th2 = blosc2.load_tensor("test.bl2")
     >>> np.array_equal(th, th2)
     True
@@ -1040,26 +1044,26 @@ def print_versions():
     import platform
 
     print("-=" * 38)
-    print("python-blosc2 version: %s" % blosc2.__version__)
-    print("Blosc version: %s" % blosc2.blosclib_version)
+    print(f"python-blosc2 version: {blosc2.__version__}")
+    print(f"Blosc version: {blosc2.blosclib_version}")
     print(f"Codecs available (including plugins): {', '.join([codec.name for codec in codecs])}")
     print("Main codec library versions:")
     for clib in sorted(clib_versions.keys()):
-        print("  %s: %s" % (clib, clib_versions[clib]))
-    print("Python version: %s" % sys.version)
+        print(f"  {clib}: {clib_versions[clib]}")
+    print(f"Python version: {sys.version}")
     (sysname, nodename, release, version, machine, processor) = platform.uname()
-    print("Platform: %s-%s-%s (%s)" % (sysname, release, machine, version))
+    print(f"Platform: {sysname}-{release}-{machine} ({version})")
     if sysname == "Linux":
         distro = os_release_pretty_name()
         if distro:
-            print("Linux dist:", distro)
+            print(f"Linux dist: {distro}")
     if not processor:
         processor = "not recognized"
-    print("Processor: %s" % processor)
-    print("Byte-ordering: %s" % sys.byteorder)
+    print(f"Processor: {processor}")
+    print(f"Byte-ordering: {sys.byteorder}")
     # Internal Blosc threading
-    print("Detected cores: %s" % blosc2.ncores)
-    print("Number of threads to use by default: %s" % blosc2.nthreads)
+    print(f"Detected cores: {blosc2.ncores}")
+    print(f"Number of threads to use by default: {blosc2.nthreads}")
     print("-=" * 38)
 
 
@@ -1111,13 +1115,14 @@ def get_chunksize(blocksize, l3_minimum=2**21, l3_maximum=2**25):
         # value only when it is an actual int.
         # Also, sometimes cpuinfo does not return a correct L3 size;
         # so in general, enforcing L3 > L2 is a good sanity check.
-        l2_cache_size = cpu_info.get("l2_cache_size", "Not found")
-        if type(l3_cache_size) is int and type(l2_cache_size) is int and l3_cache_size > l2_cache_size:
-            chunksize = l3_cache_size
+        if isinstance(l3_cache_size, int):
+            l2_cache_size = cpu_info.get("l2_cache_size", "Not found")
+            if isinstance(l2_cache_size, int) and l3_cache_size > l2_cache_size:
+                chunksize = l3_cache_size
     else:
         # Chunksize should be at least the size of L2
         l2_cache_size = cpu_info.get("l2_cache_size", "Not found")
-        if type(l2_cache_size) is int and l2_cache_size > chunksize:
+        if isinstance(l2_cache_size, int) and l2_cache_size > chunksize:
             chunksize = l2_cache_size
 
     # Ensure a minimum size
@@ -1138,7 +1143,7 @@ def compute_partition(nitems, parts, maxs, blocks=False):
     if nitems == 0:
         raise ValueError("partitions with 0 dims are not supported")
     parts = list(parts)
-    while math.prod(parts) <= nitems:
+    while math.prod(parts) < nitems:
         nitems_prev = math.prod(parts)
         # Increase dims starting from the latest
         for i in reversed(range(len(parts))):
@@ -1147,13 +1152,9 @@ def compute_partition(nitems, parts, maxs, blocks=False):
                     "blocks should be smaller than chunks or shape in any dim!"
                     " If you do want this blocks, please specify a chunks too."
                 )
-            if nitems_prev > nitems:
-                break
-            if parts[i] * 2 <= maxs[i]:
-                if math.prod(parts) * 2 <= nitems:
-                    parts[i] *= 2
-            else:
-                parts[i] = maxs[i]
+            new_part = parts[i] * 2 if parts[i] * 2 <= maxs[i] else maxs[i]
+            if math.prod(parts) // parts[i] * new_part <= nitems:
+                parts[i] = new_part
         nitems_new = math.prod(parts)
         if nitems_new == nitems_prev:
             # Not progressing anymore
@@ -1227,8 +1228,12 @@ def compute_chunks_blocks(shape, chunks=None, blocks=None, dtype=np.uint8, **kwa
                     cparams2["filters"][i] = blosc2.Filter.NOFILTER
         else:
             cparams2 = cparams
-        src = blosc2.compress2(np.zeros(nitems, dtype="V%d" % itemsize), **cparams2)
+        # Force STUNE to get a hint on the blocksize
+        aux_tuner = cparams2["tuner"] if "tuner" in cparams2 else blosc2.Tuner.STUNE
+        cparams2["tuner"] = blosc2.Tuner.STUNE
+        src = blosc2.compress2(np.zeros(nitems, dtype=f"V{itemsize}"), **cparams2)
         _, _, blocksize = blosc2.get_cbuffer_sizes(src)
+        cparams2["tuner"] = aux_tuner
         # Starting point for the guess
         if chunks is None:
             blocks = [1 if shape[i] == 1 else 2 for i in range(len(shape))]
@@ -1288,6 +1293,8 @@ def compress2(src, **kwargs):
                 The sequence of filters. Default: {0, 0, 0, 0, 0, :py:obj:`Filter.SHUFFLE <Filter>`}.
             filters_meta: list
                 The metadata for filters. Default: `{0, 0, 0, 0, 0, 0}`.
+            tuner: :class:`Tuner`
+                The tuner to use. Default: :py:obj:`Tuner.STUNE <Tuner>`.
 
     Returns
     -------
@@ -1365,6 +1372,8 @@ def remove_urlpath(path):
     None
     """
     if path is not None:
+        if isinstance(path, pathlib.PurePath):
+            path = str(path)
         path = path.encode("utf-8") if isinstance(path, str) else path
         blosc2_ext.remove_urlpath(path)
 
@@ -1393,8 +1402,32 @@ def schunk_from_cframe(cframe, copy=False):
     return blosc2_ext.schunk_from_cframe(cframe, copy)
 
 
+def ndarray_from_cframe(cframe, copy=False):
+    """Create a :ref:`NDArray <NDArray>` instance out of a contiguous frame buffer.
+
+    Parameters
+    ----------
+    cframe: bytes /str
+        The bytes object containing the in-memory cframe.
+    copy: bool
+        Whether to internally do a copy or not. If `False`,
+        the user is responsible for keeping a reference to `cframe`.
+
+    Returns
+    -------
+    out: :ref:`NDArray <NDArray>`
+        A new :ref:`NDArray <NDArray>` containing the data passed.
+
+    See Also
+    --------
+    :func:`~blosc2.NDArray.to_cframe`
+
+    """
+    return blosc2_ext.ndarray_from_cframe(cframe, copy)
+
+
 def register_codec(codec_name, id, encoder=None, decoder=None, version=1):
-    """Register an user defined codec.
+    """Register a user defined codec.
 
     Parameters
     ----------
@@ -1464,12 +1497,12 @@ def register_codec(codec_name, id, encoder=None, decoder=None, version=1):
         id = 180
         blosc2.register_codec(codec_name, id, encoder, decoder)
     """
-    if id in blosc2.ucodecs_registry.keys():
+    if id in blosc2.ucodecs_registry:
         raise ValueError("Id already in use")
     blosc2_ext.register_codec(codec_name, id, encoder, decoder, version)
 
 
-def register_filter(id, forward, backward):
+def register_filter(id, forward=None, backward=None, name=None):
     """Register an user defined filter.
 
     Parameters
@@ -1479,10 +1512,15 @@ def register_filter(id, forward, backward):
     forward: Python function
         This will receive an input to apply the filter as a ndarray of dtype uint8, an output to fill
         as a ndarray of dtype uint8, the filter meta and the corresponding `SChunk` instance.
+        If None then the filter name indicates a dynamic plugin which must be installed.
     backward: Python function
         This will receive an input as a ndarray of dtype uint8, an output to fill
         as a ndarray of dtype uint8, the filter meta and the `SChunk` instance.
-
+        If None then the filter name indicates a dynamic plugin which must be installed.
+    name: str
+        The filter name.
+        If both `forward`and `backward` are None, this parameter must be passed to correctly
+        load the dynamic filter.
     Returns
     -------
     out: None
@@ -1520,6 +1558,6 @@ def register_filter(id, forward, backward):
         id = 160
         blosc2.register_filter(id, forward, backward)
     """
-    if id in blosc2.ufilters_registry.keys():
+    if id in blosc2.ufilters_registry:
         raise ValueError("Id already in use")
-    blosc2_ext.register_filter(id, forward, backward)
+    blosc2_ext.register_filter(id, forward, backward, name)
